@@ -19,8 +19,9 @@ ShuttleGui.
 
 *//*******************************************************************/
 
-#include "../Audacity.h"
+
 #include "AudacityCommand.h"
+#include "MemoryX.h"
 
 #include "CommandContext.h"
 
@@ -35,41 +36,22 @@ ShuttleGui.
 #include <wx/utils.h>
 #include <wx/log.h>
 
-#include "audacity/ConfigInterface.h"
+#include "ConfigInterface.h"
 
-#include "../Shuttle.h"
+#include "../ShuttleAutomation.h"
 #include "../ShuttleGui.h"
 #include "../widgets/ProgressDialog.h"
 #include "../widgets/HelpSystem.h"
 #include "../widgets/AudacityMessageBox.h"
+#include "../widgets/VetoDialogHook.h"
 
 #include <unordered_map>
 
-namespace {
-
-AudacityCommand::VetoDialogHook &GetVetoDialogHook()
-{
-   static AudacityCommand::VetoDialogHook sHook = nullptr;
-   return sHook;
-}
-
-}
-
-auto AudacityCommand::SetVetoDialogHook( VetoDialogHook hook )
-   -> VetoDialogHook
-{
-   auto &theHook = GetVetoDialogHook();
-   auto result = theHook;
-   theHook = hook;
-   return result;
-}
-
 AudacityCommand::AudacityCommand()
 {
    mProgress = NULL;
    mUIParent = NULL;
    mUIDialog = NULL;
-   mUIDebug = false;
    mIsBatch = false;
    mNeedsInit = true;
 }
@@ -81,9 +63,9 @@ AudacityCommand::~AudacityCommand()
 }
 
 
-PluginPath AudacityCommand::GetPath(){        return BUILTIN_GENERIC_COMMAND_PREFIX + GetSymbol().Internal(); }
-VendorSymbol AudacityCommand::GetVendor(){      return XO("Audacity");}
-wxString AudacityCommand::GetVersion(){     return AUDACITY_VERSION_STRING;}
+PluginPath AudacityCommand::GetPath() const {        return BUILTIN_GENERIC_COMMAND_PREFIX + GetSymbol().Internal(); }
+VendorSymbol AudacityCommand::GetVendor() const {      return XO("Audacity");}
+wxString AudacityCommand::GetVersion()  const {     return AUDACITY_VERSION_STRING;}
 
 
 bool AudacityCommand::Init(){
@@ -91,7 +73,7 @@ bool AudacityCommand::Init(){
       return true;
    mNeedsInit = false;
    ShuttleDefaults DefaultSettingShuttle;
-   return DefineParams( DefaultSettingShuttle );
+   return VisitSettings( DefaultSettingShuttle );
 }
 
 bool AudacityCommand::ShowInterface(wxWindow *parent, bool WXUNUSED(forceModal))
@@ -115,8 +97,7 @@ bool AudacityCommand::ShowInterface(wxWindow *parent, bool WXUNUSED(forceModal))
    mUIDialog->SetMinSize(mUIDialog->GetSize());
 
    // The Screenshot command might be popping this dialog up, just to capture it.
-   auto hook = GetVetoDialogHook();
-   if( hook && hook( mUIDialog ) )
+   if ( VetoDialogHook::Call( mUIDialog ) )
       return false;
 
    bool res = mUIDialog->ShowModal() != 0;
@@ -136,7 +117,7 @@ wxDialog *AudacityCommand::CreateUI(wxWindow *parent, AudacityCommand * WXUNUSED
    return NULL;
 }
 
-bool AudacityCommand::GetAutomationParameters(wxString & parms)
+bool AudacityCommand::SaveSettingsAsString(wxString & parms)
 {
    CommandParameters eap;
 
@@ -147,14 +128,14 @@ bool AudacityCommand::GetAutomationParameters(wxString & parms)
 
    ShuttleGetAutomation S;
    S.mpEap = &eap;
-   bool bResult = DefineParams( S );
+   bool bResult = VisitSettings( S );
    wxASSERT_MSG( bResult, "You did not define DefineParameters() for this command" );
    static_cast<void>(bResult); // fix unused variable warning in release mode
 
    return eap.GetParameters(parms);
 }
 
-bool AudacityCommand::SetAutomationParameters(const wxString & parms)
+bool AudacityCommand::LoadSettingsFromString(const wxString & parms)
 {
    wxString preset = parms;
 
@@ -162,7 +143,7 @@ bool AudacityCommand::SetAutomationParameters(const wxString & parms)
    ShuttleSetAutomation S;
 
    S.SetForWriting( &eap );
-   bool bResult = DefineParams( S );
+   bool bResult = VisitSettings( S );
    wxASSERT_MSG( bResult, "You did not define DefineParameters() for this command" );
    static_cast<void>(bResult); // fix unused variable warning in release mode
    if (!S.bOK)
@@ -238,6 +219,16 @@ bool AudacityCommand::TransferDataFromWindow()
    return true;
 }
 
+bool AudacityCommand::VisitSettings( SettingsVisitor & )
+{
+   return false;
+}
+
+bool AudacityCommand::VisitSettings( ConstSettingsVisitor & )
+{
+   return false;
+}
+
 int AudacityCommand::MessageBox(
    const TranslatableString& message, long style,
    const TranslatableString &titleStr)
