@@ -19,7 +19,7 @@
 *//*******************************************************************/
 
 
-#include "../Audacity.h"
+
 #include "FindClipping.h"
 #include "LoadEffects.h"
 
@@ -27,7 +27,6 @@
 
 #include <wx/intl.h>
 
-#include "../Shuttle.h"
 #include "../ShuttleGui.h"
 #include "../widgets/valnum.h"
 #include "../widgets/AudacityMessageBox.h"
@@ -35,11 +34,13 @@
 #include "../LabelTrack.h"
 #include "../WaveTrack.h"
 
-// Define keys, defaults, minimums, and maximums for the effect parameters
-//
-//     Name    Type  Key                     Def   Min   Max      Scale
-Param( Start,  int,  wxT("Duty Cycle Start"), 3,    1,    INT_MAX, 1   );
-Param( Stop,   int,  wxT("Duty Cycle End"),   3,    1,    INT_MAX, 1   );
+const EffectParameterMethods& EffectFindClipping::Parameters() const
+{
+   static CapturedParameters<EffectFindClipping,
+      Start, Stop
+   > parameters;
+   return parameters;
+}
 
 const ComponentInterfaceSymbol EffectFindClipping::Symbol
 { XO("Find Clipping") };
@@ -48,8 +49,7 @@ namespace{ BuiltinEffectsModule::Registration< EffectFindClipping > reg; }
 
 EffectFindClipping::EffectFindClipping()
 {
-   mStart = DEF_Start;
-   mStop = DEF_Stop;
+   Parameters().Reset(*this);
 }
 
 EffectFindClipping::~EffectFindClipping()
@@ -58,60 +58,34 @@ EffectFindClipping::~EffectFindClipping()
 
 // ComponentInterface implementation
 
-ComponentInterfaceSymbol EffectFindClipping::GetSymbol()
+ComponentInterfaceSymbol EffectFindClipping::GetSymbol() const
 {
    return Symbol;
 }
 
-TranslatableString EffectFindClipping::GetDescription()
+TranslatableString EffectFindClipping::GetDescription() const
 {
    return XO("Creates labels where clipping is detected");
 }
 
-wxString EffectFindClipping::ManualPage()
+ManualPageID EffectFindClipping::ManualPage() const
 {
-   return wxT("Find_Clipping");
+   return L"Find_Clipping";
 }
 
 // EffectDefinitionInterface implementation
 
-EffectType EffectFindClipping::GetType()
+EffectType EffectFindClipping::GetType() const
 {
    return EffectTypeAnalyze;
 }
 
-// EffectClientInterface implementation
-bool EffectFindClipping::DefineParams( ShuttleParams & S ){
-   S.SHUTTLE_PARAM( mStart, Start );
-   S.SHUTTLE_PARAM( mStop, Stop );
-   return true;
-}
-
-bool EffectFindClipping::GetAutomationParameters(CommandParameters & parms)
-{
-   parms.Write(KEY_Start, mStart);
-   parms.Write(KEY_Stop, mStop);
-
-   return true;
-}
-
-bool EffectFindClipping::SetAutomationParameters(CommandParameters & parms)
-{
-   ReadAndVerifyInt(Start);
-   ReadAndVerifyInt(Stop);
-
-   mStart = Start;
-   mStop = Stop;
-
-   return true;
-}
-
 // Effect implementation
 
-bool EffectFindClipping::Process()
+bool EffectFindClipping::Process(EffectInstance &, EffectSettings &)
 {
    std::shared_ptr<AddedAnalysisTrack> addedTrack;
-   Optional<ModifiedAnalysisTrack> modifiedTrack;
+   std::optional<ModifiedAnalysisTrack> modifiedTrack;
    const wxString name{ _("Clipping") };
 
    auto clt = *inputTracks()->Any< const LabelTrack >().find_if(
@@ -198,7 +172,7 @@ bool EffectFindClipping::ProcessOne(LabelTrack * lt,
 
          block = limitSampleBufferSize( blockSize, len - s );
 
-         wt->Get((samplePtr)buffer.get(), floatSample, start + s, block);
+         wt->GetFloats(buffer.get(), start + s, block);
          ptr = buffer.get();
       }
 
@@ -240,30 +214,42 @@ bool EffectFindClipping::ProcessOne(LabelTrack * lt,
    return bGoodResult;
 }
 
-void EffectFindClipping::PopulateOrExchange(ShuttleGui & S)
+std::unique_ptr<EffectUIValidator> EffectFindClipping::PopulateOrExchange(
+   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &access)
 {
+   DoPopulateOrExchange(S, access);
+   return nullptr;
+}
+
+void EffectFindClipping::DoPopulateOrExchange(
+   ShuttleGui & S, EffectSettingsAccess &access)
+{
+   mpAccess = access.shared_from_this();
    S.StartMultiColumn(2, wxALIGN_CENTER);
    {
-      S.Validator<IntegerValidator<int>>(
-            &mStart, NumValidatorStyle::DEFAULT, MIN_Start)
+      S
+         .Validator<IntegerValidator<int>>(
+            &mStart, NumValidatorStyle::DEFAULT, Start.min)
          .TieTextBox(XXO("&Start threshold (samples):"), mStart, 10);
 
-      S.Validator<IntegerValidator<int>>(
-            &mStop, NumValidatorStyle::DEFAULT, MIN_Stop)
+      S
+         .Validator<IntegerValidator<int>>(
+            &mStop, NumValidatorStyle::DEFAULT, Stop.min)
          .TieTextBox(XXO("St&op threshold (samples):"), mStop, 10);
    }
    S.EndMultiColumn();
 }
 
-bool EffectFindClipping::TransferDataToWindow()
+bool EffectFindClipping::TransferDataToWindow(const EffectSettings &)
 {
    ShuttleGui S(mUIParent, eIsSettingToDialog);
-   PopulateOrExchange(S);
+   // To do: eliminate this and just use validators for controls
+   DoPopulateOrExchange(S, *mpAccess);
 
    return true;
 }
 
-bool EffectFindClipping::TransferDataFromWindow()
+bool EffectFindClipping::TransferDataFromWindow(EffectSettings &)
 {
    if (!mUIParent->Validate())
    {
@@ -271,7 +257,8 @@ bool EffectFindClipping::TransferDataFromWindow()
    }
 
    ShuttleGui S(mUIParent, eIsGettingFromDialog);
-   PopulateOrExchange(S);
+   // To do: eliminate this and just use validators for controls
+   DoPopulateOrExchange(S, *mpAccess);
 
    return true;
 }
