@@ -21,10 +21,10 @@
       
 **********************************************************************/
 
-#include "../../Audacity.h"
+
 #include "VSTControlOSX.h"
 
-#include "../../MemoryX.h"
+#include <memory>
 
 @interface VSTView : NSView
 {
@@ -129,7 +129,7 @@ bool VSTControl::Create(wxWindow *parent, VSTEffectLink *link)
 
 void VSTControl::CreateCocoa()
 {
-   if ((mLink->callDispatcher(effCanDo, 0, 0, (void *) "hasCockosViewAsConfig", 0.0) & 0xffff0000) != 0xbeef0000)
+   if ((mLink->callDispatcher(effCanDo, 0, 0, (void *) "hasCockosViewAsConfig", 0.0) & 0xffff0000) == 0xbeef0000)
    {
       return;
    }
