@@ -11,13 +11,14 @@
 
 **********************************************************************/
 
-#include "../../Audacity.h" // for USE_* macros
+
 
 #if defined(USE_VAMP)
 
 #include <vamp-hostsdk/PluginLoader.h>
 
 #include "../Effect.h"
+#include "MemoryX.h"
 
 class wxStaticText;
 class wxSlider;
@@ -26,12 +27,14 @@ class wxCheckBox;
 class wxTextCtrl;
 class LabelTrack;
 
+using Floats = ArrayOf<float>;
+
 #define VAMPEFFECTS_VERSION wxT("1.0.0.0")
 /* i18n-hint: Vamp is the proper name of a software protocol for sound analysis.
    It is not an abbreviation for anything.  See http://vamp-plugins.org */
 #define VAMPEFFECTS_FAMILY XO("Vamp")
 
-class VampEffect final : public Effect
+class VampEffect final : public StatefulEffect
 {
 public:
    VampEffect(std::unique_ptr<Vamp::Plugin> &&plugin,
@@ -42,33 +45,35 @@ public:
 
    // ComponentInterface implementation
 
-   PluginPath GetPath() override;
-   ComponentInterfaceSymbol GetSymbol() override;
-   VendorSymbol GetVendor() override;
-   wxString GetVersion() override;
-   TranslatableString GetDescription() override;
+   PluginPath GetPath() const override;
+   ComponentInterfaceSymbol GetSymbol() const override;
+   VendorSymbol GetVendor() const override;
+   wxString GetVersion() const override;
+   TranslatableString GetDescription() const override;
 
    // EffectDefinitionInterface implementation
 
-   EffectType GetType() override;
-   EffectFamilySymbol GetFamily() override;
-   bool IsInteractive() override;
-   bool IsDefault() override;
+   EffectType GetType() const override;
+   EffectFamilySymbol GetFamily() const override;
+   bool IsInteractive() const override;
+   bool IsDefault() const override;
 
-   // EffectClientInterface implementation
+   bool SaveSettings(
+      const EffectSettings &settings, CommandParameters & parms) const override;
+   bool LoadSettings(
+      const CommandParameters & parms, EffectSettings &settings) const override;
 
-   unsigned GetAudioInCount() override;
-   bool GetAutomationParameters(CommandParameters & parms) override;
-   bool SetAutomationParameters(CommandParameters & parms) override;
+   unsigned GetAudioInCount() const override;
 
    // Effect implementation
 
    bool Init() override;
-   bool Process() override;
-   void End() override;
-   void PopulateOrExchange(ShuttleGui & S) override;
-   bool TransferDataToWindow() override;
-   bool TransferDataFromWindow() override;
+   bool Process(EffectInstance &instance, EffectSettings &settings) override;
+   std::unique_ptr<EffectUIValidator> PopulateOrExchange(
+      ShuttleGui & S, EffectInstance &instance, EffectSettingsAccess &access)
+   override;
+   bool TransferDataToWindow(const EffectSettings &settings) override;
+   bool TransferDataFromWindow(EffectSettings &settings) override;
 
 private:
    // VampEffect implementation
