@@ -14,29 +14,31 @@
 #ifndef __AUDACITY_EDIT_TOOLBAR__
 #define __AUDACITY_EDIT_TOOLBAR__
 
-#include "../Experimental.h"
-
 #include <wx/defs.h>
 
 #include "ToolBar.h"
 
 class wxCommandEvent;
 class wxDC;
+class wxGridSizer;
 class wxImage;
 class wxWindow;
 
 class AButton;
 
 enum {
-   ETBCutID,
-   ETBCopyID,
-   ETBPasteID,
+   ETBZoomInID,
+   ETBZoomOutID,
+#ifdef EXPERIMENTAL_ZOOM_TOGGLE_BUTTON
+   ETBZoomToggleID,
+#endif
+
+   ETBZoomSelID,
+   ETBZoomFitID,
+
    ETBTrimID,
    ETBSilenceID,
 
-   ETBUndoID,
-   ETBRedoID,
-
 #ifdef EXPERIMENTAL_SYNC_LOCK
    //Undefined, so no sync-lock on/off button.
    //#define OPTION_SYNC_LOCK_BUTTON
@@ -46,18 +48,8 @@ enum {
    ETBSyncLockID,
 #endif
 
-   ETBZoomInID,
-   ETBZoomOutID,
-#ifdef EXPERIMENTAL_ZOOM_TOGGLE_BUTTON
-   ETBZoomToggleID,
-#endif
-
-   ETBZoomSelID,
-   ETBZoomFitID,
-
-#if defined(EXPERIMENTAL_EFFECTS_RACK)
-   ETBEffectsID,
-#endif
+   ETBUndoID,
+   ETBRedoID,
 
    ETBNumButtons
 };
@@ -102,6 +94,8 @@ class EditToolBar final : public ToolBar {
 
    AButton *mButtons[ETBNumButtons];
 
+   wxGridSizer* mToolSizer;
+
    wxImage *upImage;
    wxImage *downImage;
    wxImage *hiliteImage;
