@@ -58,7 +58,7 @@ class TipWindow;
 // which uses this class, is below.
 //
 
-class LWSlider
+class AUDACITY_DLL_API LWSlider
 {
    friend class ASlider;
    friend class ASliderAx;
@@ -75,6 +75,10 @@ class LWSlider
             float stepValue,
             bool canUseShift,
             int style,
+            bool showlabels=true,
+            bool drawticks=true,
+            bool drawtrack=true,
+            bool alwayshidetip=false,
             bool heavyweight=false,
             bool popup=true,
             int orientation = wxHORIZONTAL); // wxHORIZONTAL or wxVERTICAL. wxVERTICAL is currently only for DB_SLIDER.
@@ -85,6 +89,10 @@ class LWSlider
             const wxPoint &pos,
             const wxSize &size,
             int style,
+            bool showlabels=true,
+            bool drawticks=true,
+            bool drawtrack=true,
+            bool alwayshidetip=false,
             bool heavyweight=false,
             bool popup=true,
             int orientation = wxHORIZONTAL); // wxHORIZONTAL or wxVERTICAL. wxVERTICAL is currently only for DB_SLIDER.
@@ -98,6 +106,10 @@ class LWSlider
              float stepValue,
              bool canUseShift,
              int style,
+             bool showlabels,
+             bool drawticks,
+             bool drawtrack,
+             bool alwayshidetip,
              bool heavyweight,
              bool popup,
              float speed,
@@ -136,14 +148,16 @@ class LWSlider
    void OnMouseEvent(wxMouseEvent & event);
    void OnKeyDown(wxKeyEvent & event);
    void Refresh();
+   void Redraw();
 
    bool ShowDialog();
    bool ShowDialog(wxPoint pos);
 
    void SetEnabled(bool enabled);
-   bool GetEnabled();
+   bool GetEnabled() const;
 
-   static void DeleteSharedTipPanel();
+   float GetMinValue() const;
+   float GetMaxValue() const;
 
    void SetParent(wxWindow *parent) { mParent = parent; }
    void SendUpdate(float newValue);
@@ -169,6 +183,12 @@ class LWSlider
    int mStyle;
    int mOrientation; // wxHORIZONTAL or wxVERTICAL. wxVERTICAL is currently only for DB_SLIDER.
 
+   bool mShowLabels;
+   bool mDrawTicks;
+   bool mDrawTrack;
+
+   bool mAlwaysHideTip;
+
    bool mHW; // is it really heavyweight (in a window)
    bool mPopup; // should display dialog on double click
 
@@ -228,7 +248,7 @@ class LWSlider
    bool mEnabled;
 };
 
-class ASlider /* not final */ : public wxPanel
+class AUDACITY_DLL_API ASlider /* not final */ : public wxPanel
 {
    friend class ASliderAx;
 
@@ -238,6 +258,10 @@ class ASlider /* not final */ : public wxPanel
 
       int style{ FRAC_SLIDER };
       wxOrientation orientation{ wxHORIZONTAL };
+      bool showLabels{ true };
+      bool drawTicks{ true };
+      bool drawTrack{ true };
+      bool alwaysHideTip{ false };
       bool popup{ true };
       bool canUseShift{ true };
       float stepValue{ STEP_CONTINUOUS };
@@ -248,6 +272,10 @@ class ASlider /* not final */ : public wxPanel
       Options& Style( int s ) { style = s; return *this; }
       Options& Orientation( wxOrientation o )
          { orientation = o; return *this; }
+      Options& ShowLabels( bool l ) { showLabels = l; return *this; }
+      Options& DrawTicks( bool t ) { drawTicks = t; return *this; }
+      Options& DrawTrack( bool t ) { drawTrack = t; return *this; }
+      Options& AlwayHideTip( bool t) { alwaysHideTip = t; return * this; }
       Options& Popup( bool p ) { popup = p; return *this; }
       Options& CanUseShift( bool c ) { canUseShift = c; return *this; }
       Options& StepValue( float v ) { stepValue = v; return *this; }
@@ -269,6 +297,8 @@ class ASlider /* not final */ : public wxPanel
 
    void SetFocusFromKbd() override;
 
+   bool SetBackgroundColour(const wxColour& colour) override;
+
    void GetScroll(float & line, float & page);
    void SetScroll(float line, float page);
 
@@ -353,6 +383,7 @@ class SliderDialog final : public wxDialogWrapper
    wxTextCtrl * mTextCtrl;
    int mStyle;
    LWSlider * mpOrigin;
+   float mValue;
 
  public:
    DECLARE_EVENT_TABLE()
